#========================================================================================
# state        private
#========================================================================================
package DE_EPAGES::AppStore::t::DeveloperAppsClient;
use base DE_EPAGES::Database::API::Test;

use DE_EPAGES::AppStore::API::Object::DeveloperAppsClient;
use DE_EPAGES::EPagesJ::API::Object::UriTemplate;
use DE_EPAGES::Test::Mock::EpagesObject;
use HTTP::Response;
use HTTP::Status qw ( :constants :is status_message );
use JSON qw ( encode_json );
use Test::MockObject;
use Test::More;

use strict;

## no critic qw(ProhibitLongLines)

use constant ALL_SCOPES => 212990;

sub test {

    my $Shop = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', {
        GUID => 'shopGUID'
    });

    my $Client = DE_EPAGES::AppStore::API::Object::DeveloperAppsClient->create( {
            BaseURL => 'http://localhost:8088/rs-test/',
            Shop    => $Shop,
        }
    );
    $Client->{'HttpClient'} = _mockHTTPClient( {
            'post'   => \&_assertCreateAppContent,
            'get'    => \&_assertGetAppContent,
            'delete' => \&_assertDeleteAppContent,
            'put'    => \&_assertSaveAppContent,
        }
    );

    # check urls
    ## no critic (ProhibitPrivateSubroutineUse)
    like( $Client->_appsUrl(),         qr#appstore/shopGUID/private-apps#,     'multiple apps url OK' );
    like( $Client->_singleAppUrl(333), qr#appstore/shopGUID/private-apps/333#, 'single app url OK' );

    # create Developer app
    my $privateApp = $Client->createApp( 'bourneIdentity' , 'callback', ALL_SCOPES);
    is( $privateApp->{'name'},                         'bourneIdentity', 'App name is set to: ' . $privateApp->{'name'} );
    is( $privateApp->{'shopGUID'},                     'shopGUID',       'App shopGUID is set to: ' . $privateApp->{'shopGUID'} );
    is( $privateApp->{'credentials'}->{'consumerKey'}, 'demo_consumer',  'demo_consumer OK' );

    # get single app
    $privateApp = $Client->getApp(333);
    is( $privateApp->{'name'}, 'bourneIdentityGET', 'App name is set to: ' . $privateApp->{'name'} );

    # get list apps
    my @aList = $Client->listApps();
    is( $aList[0][0]->{'name'}, 'bourneIdentityGETLIST', 'App list first name is :' . $aList[0][0]->{'name'} );
    is( $aList[0][1]->{'name'}, 'AlienThemeGETLIST',     'App list first name is :' . $aList[0][1]->{'name'} );

    # delete app
    $Client->deleteApp(333);
}

sub _mockHTTPClient {
    my ($hMockMethods) = @_;
    my $mockRemote = Test::MockObject->new();
    $mockRemote->set_isa('DE_EPAGES::EPagesJ::API::Object::HTTPClient');
    $mockRemote->fake_module( 'DE_EPAGES::EPagesJ::API::Object::HTTPClient', %$hMockMethods );
    while ( my ( $method, $handler ) = each %$hMockMethods ) {
        $mockRemote->mock( $method, $handler );
    }
    return $mockRemote;
}

sub _assertGetAppContent {
    shift;
    my ( $URL, $hHeader, $Content ) = @_;

    if ( $URL->abs eq 'http://localhost:8088/rs-test/appstore/shopGUID/private-apps/333' ) {

        return HTTP::Response->new( HTTP_OK, status_message(HTTP_OK), undef,
            '{ "id": 333, "name": "bourneIdentityGET","callbackUrl": "callback", "shopGUID": "shopGUID" }'
        );
    }
    return HTTP::Response->new( HTTP_OK, status_message(HTTP_OK), undef,
        '[{ "id": 333, "name": "bourneIdentityGETLIST","callbackUrl": "callback", "shopGUID": "shopGUID" }, { "id": 444, "name": "AlienThemeGETLIST","callbackUrl": "callback", "shopGUID": "shopGUID" }]'
    );
}

sub _assertDeleteAppContent {
    shift;
    my ( $URL, $hHeader, $Content ) = @_;
    is( $URL->abs, 'http://localhost:8088/rs-test/appstore/shopGUID/private-apps/333', 'path for delete' );
    return HTTP::Response->new( HTTP_NO_CONTENT, status_message(HTTP_NO_CONTENT), undef, undef );
}

sub _assertCreateAppContent {
    shift;
    my ( $URL, $hHeader, $Content ) = @_;
    is( $URL->abs, 'http://localhost:8088/rs-test/appstore/shopGUID/private-apps', 'post path for create private app' );
    is(
        $Content,
        encode_json( {
                'name'            => 'bourneIdentity',
                'callbackUrl'     => 'callback',
                'scopes'          => ALL_SCOPES,
                'shopGUID'        => 'shopGUID',
            }
        ),
        'post contents'
    );
    return HTTP::Response->new( HTTP_CREATED, status_message(HTTP_CREATED), undef,
        <<'EOL'
{ "id": 333,
    "name": "bourneIdentity",
    "callbackUrl": "callback",
    "shopGUID": "shopGUID",
    "credentials" : {
           "type": "oauth1",
           "consumerKey": "demo_consumer",
           "consumerSecret": "demo_consumer_key",
           "accessTokenKey": "demo_token",
           "accessTokenSecret": "demo_token_key"
    }
}
EOL
    );
}

sub _assertSaveAppContent {
    shift;
    my ( $URL, $hHeader, $Content ) = @_;
    is( $URL->abs, 'http://localhost:8088/rs-test/appstore/shopGUID/private-apps/333', 'put path for save private app' );
    is(
        $Content,
        encode_json( {
                'id'              => 333,
                'callbackUrl'     => 'callback',
                'name'            => 'bourneIdentitySAVE',
                'shopGUID'        => 'shopGUID',
            }
        ),
        'put contents'
    );
    return HTTP::Response->new( HTTP_CREATED, status_message(HTTP_CREATED), undef,
        <<'EOL'
{ "id": 333,
    "callbackUrl" : "callback",
    "name": "bourneIdentitySAVE",
    "shopGUID": "shopGUID",
    "credentials" : { 
           "type": "oauth1",
           "consumerKey": "demo_consumer",
           "consumerSecret": "demo_consumer_key",
           "accessTokenKey": "demo_token",
           "accessTokenSecret": "demo_token_key"
       }
}
EOL
    );
}

__PACKAGE__->runScript;
done_testing();
