#========================================================================================
# package      DE_EPAGES::Ipayment::t::CGI_ELV_Positive_si
# state        public
#----------------------------------------------------------------------------------------
# description  System Integration Test for ELV Transactions
#========================================================================================
package DE_EPAGES::Ipayment::t::CGI_ELV_Positive_si;
use base DE_EPAGES::Shop::API::TestImportXML;

use strict;

use Readonly;
use URI;
use URI::QueryParam;

use DE_EPAGES::Ipayment::API::CGI::AuthOrPreauth;
use DE_EPAGES::Ipayment::API::Order::Payment;
use DE_EPAGES::Ipayment::API::Session;
use DE_EPAGES::Ipayment::API::SOAP::Request::HashBuilder;

use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );

use Test::More;
use Test::MockObject::Extends;

Readonly my $SubTestName => 'CGI_ELV_Positive_si.t';


#----------------------------------------------------------------------------------------
# Wrap for DE_EPAGES::Shop::API::TestImportXML
# You need to override this methods in other to use
#----------------------------------------------------------------------------------------
sub shopAlias {
    return 'CgiElvShop';
}
sub importXMLPath {
    return 'ShopXMLs/ShopWithIpaymentELV.xml';
}

#----------------------------------------------------------------------------------------

sub test {
    my $self = shift;

    my $Order;
    Readonly my $ORDERPATH => $self->shopPath().'/Customers/TestCustomer/Orders/1234-5678';

    # Preauth and capture
    $Order = LoadObjectByPath($ORDERPATH);
    _createSession($Order);
    _preauthTransaction($Order);
    _captureTransaction($Order);

    # Preauth and cancel
    $Order = LoadObjectByPath($ORDERPATH);
    _createSession($Order);
    _preauthTransaction($Order);
    _cancelTransaction($Order);

    done_testing();
}
#----------------------------------------------------------------------------------------


sub _preauthTransaction() {
    my ($Order) = @_;

    my $AuthOrPreauth = DE_EPAGES::Ipayment::API::CGI::AuthOrPreauth->new();
    my $ReturnParams = $AuthOrPreauth->getRequestParameters($Order);
    my $hRequestParams = $ReturnParams->{'HiddenInputs'};
    my $PaymentUrl = $ReturnParams->{'PaymentUrl'};

    $hRequestParams->{'silent'} = '1';
    $hRequestParams->{'bank_country'} = 'DE';
    $hRequestParams->{'bank_name'} = 'Bundesbank';
    $hRequestParams->{'bank_code'} = '10000000';
    $hRequestParams->{'bank_accountnumber'} = '12345678';

    my $UserAgent = LWP::UserAgent->new();
    my $Response = $UserAgent->post( $PaymentUrl, $hRequestParams );

    my $ReturnUrl = $Response->{'_content'};
    my $RedirectURI = URI->new($ReturnUrl);
    my $ResponseParameters = $RedirectURI->query_form_hash();

    is($ResponseParameters->{'trx_typ'}, 'preauth', "$SubTestName - test if transaction was done with preauth");
    is($ResponseParameters->{'trx_paymentmethod'}, 'ELV', "$SubTestName - test if paymentmethod was ELV");
    #is($ResponseParameters->{'ret_status'}, 'SUCCESS">Silent-Redirect</a><br', "$SubTestName - test if return status was success");

    # Set TransID and TransStatus to prepare the order for capture or cancel.
    my $LineItemPaymentIpayment = $Order->container()->get('Payment');
    $LineItemPaymentIpayment->set({
        'TransID' => $ResponseParameters->{'ret_trx_number'},
        'TransStatus' => 'STATUS_ACCEPTED',
    });

}



sub _captureTransaction() {
    my ($Order) = @_;

    my $TransStatus;
    $TransStatus = $Order->container()->get('Payment')->get('TransStatus');
    is($TransStatus, 'STATUS_ACCEPTED', "$SubTestName - test if transaction status is accepted");

    my $Payment = DE_EPAGES::Ipayment::API::Order::Payment->new();
    $Payment->capture($Order);

    $TransStatus = $Order->container()->get('Payment')->get('TransStatus');
    #is($TransStatus, 'STATUS_PAID', "$SubTestName - test if transaction status is paid");

}



sub _cancelTransaction() {
    my ($Order) = @_;

    my $TransStatus;
    $TransStatus = $Order->container()->get('Payment')->get('TransStatus');
    is($TransStatus, 'STATUS_ACCEPTED', "$SubTestName - test if transaction status is accepted");

    my $Payment = DE_EPAGES::Ipayment::API::Order::Payment->new();
    $Payment->cancel($Order);

    $TransStatus = $Order->container()->get('Payment')->get('TransStatus');
    #is($TransStatus, 'STATUS_CANCELLED', "$SubTestName - test if transaction status is canceled");

}



sub _createSession() {
    my ($Order) = @_;

    #prepare
    my $Session = DE_EPAGES::Ipayment::API::Session->new();
    $Session = Test::MockObject::Extends->new( $Session );
    DisableDoubleTransactionCheck($Session);

    #request
    my $Status = $Session->createSessionAtIpaymentAndStoreIdInDatabase($Order);

    ok($Status, "$SubTestName - tests if the session has be created");
    my $SessionID = $Order->container()->get('Payment')->get('SessionID');
    ok(isValid($SessionID), "$SubTestName - tests if the session is stored in the database");
}



sub DisableDoubleTransactionCheck() {
    my ($Session ) = @_;

    $Session->mock('_createHashbuilder', sub {
        my $self = shift;
        my ($DatabaseBasketOrOrder) = @_;
        my $HashBuilder = DE_EPAGES::Ipayment::API::SOAP::Request::HashBuilder->new($DatabaseBasketOrOrder);
        my $MockedHashBuilder = Test::MockObject::Extends->new( $HashBuilder );

        $MockedHashBuilder->mock( '_assembleOptions', sub {
            my $self = shift;
            return {
                    'fromIp' => '',
                    'advancedStrictIdCheck' => '0',
                    'checkFraudattack' => '',
                    'checkDoubleTrx' => '0',
                    'errorLang' => '',
                    'trxLongsafe' => '',
                    'otherOptions' => {
                        'option' => {
                            'key' => '',
                            'value' => ''
                        }
                    },
                    'browserData' => {
                        'browserUserAgent' => '',
                        'browserAcceptHeaders' => '',
                    },
                    'clientData' => {
                        'clientName' => 'ePages',
                        'clientVersion' => '',
                    }
                };
        } );
        return $MockedHashBuilder;
    });
    return undef;
}

__PACKAGE__->runScript();

1;