#========================================================================================
# package      DE_EPAGES::Order::t::RestApiIntegrationTest
# base         DE_EPAGES::Shop::API::TestImportXML
# state        public
#----------------------------------------------------------------------------------------
# description  Integration test for RestApi resources.
#               Checks if requests are matched by the correct resources.
#========================================================================================
package DE_EPAGES::Order::t::RestApiIntegrationTest;
use base DE_EPAGES::Shop::API::TestImportXML;

use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );
use DE_EPAGES::WebService::API::ResourceRegistry;
use HTTP::Request;
use Test::Exception;
use Test::More;

use strict;



sub shopAlias {
    return 'TestShop';
}



sub importXMLPath {
    return 'ShopXMLs/Shop_for_RestApiIntegration_test.xml';
}



sub test {
    my $self = shift;

    $self->test_post_cart();

    $self->test_get_cartById();
    $self->test_patch_cartById();

    $self->test_get_checkoutButton();

    $self->test_get_orders();

    $self->test_get_orderById();
    $self->test_put_orderById();
    $self->test_patch_orderById();

    $self->test_post_cart_to_order();

    $self->test_get_sales();

    done_testing();
}



sub test_post_cart {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $Request = $self->_createRequest('POST', '/api/carts/', 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Order::API::Resource::CartCreationResource', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'post', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Shop::API::Object::Shop', "$SubTestName - test if object was extracted correctly");
}



sub test_get_cartById {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $BasketPath = $self->shopPath().'/AnonymousUsers/User123/Baskets/Basket314';
    my $Basket = LoadObjectByPath($BasketPath);
    my $Request = $self->_createRequest('GET', '/api/carts/'.$Basket->guid().'?includeTax=false', 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Order::API::Resource::Cart', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'get', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Order::API::Object::Basket', "$SubTestName - test if object was extracted correctly");
    is($aParams->[1], 'false', "$SubTestName - test if the includeTax was extracted correctly");
}



sub test_patch_cartById {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $BasketPath = $self->shopPath().'/AnonymousUsers/User123/Baskets/Basket314';
    my $Basket = LoadObjectByPath($BasketPath);
    my $Request = $self->_createRequest('PATCH', '/api/carts/'.$Basket->guid(), 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Order::API::Resource::Cart', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'patch', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Order::API::Object::Basket', "$SubTestName - test if object was extracted correctly");
}


sub test_get_checkoutButton {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $BasketPath = $self->shopPath().'/AnonymousUsers/User123/Baskets/Basket314';
    my $Basket = LoadObjectByPath($BasketPath);
    my $Request = $self->_createRequest('GET', '/api/carts/'.$Basket->guid().'/checkoutbutton', 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Order::API::Resource::CartCheckoutButton', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'get', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Order::API::Object::Basket', "$SubTestName - test if object was extracted correctly");
}


sub test_get_orders {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $Request = $self->_createRequest('GET', '/api/orders/?locale=de_DE&page=3&resultsPerPage=5', 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Order::API::Resource::OrderCollection', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'get', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Shop::API::Object::Shop', "$SubTestName - test if object was extracted correctly");
    is($aParams->[1], 'de_DE', "$SubTestName - test if the locale was extracted correctly");
    is($aParams->[2], 5, "$SubTestName - test if page size was extracted correctly");
    is($aParams->[3], 3, "$SubTestName - test if page was extracted correctly");
}



sub test_get_orderById {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $OrderPath = $self->shopPath().'/Customers/TestCustomer/Orders/1234-5678';
    my $Order = LoadObjectByPath($OrderPath);
    my $Request = $self->_createRequest('GET', '/api/orders/'.$Order->guid().'?locale=de_DE', 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Order::API::Resource::OrderResource', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'get', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Order::API::Object::CustomerOrder', "$SubTestName - test if object was extracted correctly");
    is($aParams->[1], 'de_DE', "$SubTestName - test if the locale was extracted correctly");
}



sub test_put_orderById {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $OrderPath = $self->shopPath().'/Customers/TestCustomer/Orders/1234-5678';
    my $Order = LoadObjectByPath($OrderPath);
    my $Request = $self->_createRequest('PUT', '/api/orders/'.$Order->guid().'?locale=de_DE', 'application/vnd.epages.v1+json', '{}');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Order::API::Resource::OrderResource', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'put', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Order::API::Object::CustomerOrder', "$SubTestName - test if object was extracted correctly");
    is($aParams->[1], 'de_DE', "$SubTestName - test if the locale was extracted correctly");
}



sub test_patch_orderById {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $OrderPath = $self->shopPath().'/Customers/TestCustomer/Orders/1234-5678';
    my $Order = LoadObjectByPath($OrderPath);
    my $Request = $self->_createRequest('PATCH', '/api/orders/'.$Order->guid().'?locale=de_DE', 'application/vnd.epages.v1+json', '{}');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Order::API::Resource::OrderResource', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'patch', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Order::API::Object::CustomerOrder', "$SubTestName - test if object was extracted correctly");
    is($aParams->[1], 'de_DE', "$SubTestName - test if the locale was extracted correctly");
}



sub test_post_cart_to_order {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $BasketPath = $self->shopPath().'/AnonymousUsers/User123/Baskets/Basket314';
    my $Basket = LoadObjectByPath($BasketPath);
    my $Request = $self->_createRequest('POST', '/api/carts/'.$Basket->guid().'/order', 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Order::API::Resource::OrderWithCustomerCreationResource', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'post', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Order::API::Object::Basket', "$SubTestName - test if object was extracted correctly");
}



sub test_get_sales {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $Request = $self->_createRequest('GET', '/api/sales/?createdAfter=2015-05-14T14:12:15&createdBefore=2015-08-03T06:04:47', 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Order::API::Resource::SaleResource', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'get', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Shop::API::Object::Shop', "$SubTestName - test if object was extracted correctly");
    is($aParams->[1], '2015-05-14T16:12:15', "$SubTestName - test if the createdAfter was extracted correctly");
    is($aParams->[2], '2015-08-03T08:04:47', "$SubTestName - test if createdBefore was extracted correctly");
}



sub _createRequest {
    my $self = shift;
    my ($Method, $Route, $Accept, $Content) = @_;

    my $URL = 'http://www.example.com'.$Route;
    my $Request = HTTP::Request->new($Method => $URL);

    if(defined $Accept) {
        $Request->header( 'Accept' => $Accept );
    }
    if(defined $Content) {
        warn "set content: $Content \n";
        $Request->content( $Content );
    }

    return $Request;
}



__PACKAGE__->runScript();

1;