#========================================================================================
# package      DE_EPAGES::PayPalPlus::t::unit::GetMerchantStatus_Repository
# state        public
#----------------------------------------------------------------------------------------
# description  unit test for GetMerchantStatus repository
#========================================================================================
package DE_EPAGES::PayPalPlus::t::unit::GetMerchantStatus_Repository;
use base DE_EPAGES::Core::API::Test;

use Test::More;
use Test::Exception;

use DE_EPAGES::PayPalPlus::API::Webservice::GetMerchantStatus::Repository;
use DE_EPAGES::Test::Mock::EpagesObject;

use strict;

sub test {
    my $self = shift;

    $self->test_buildRequest_successful();
    $self->test_buildRequest_missingProviderPayPalAccountId();

    $self->test_writeResponse_successful();
    $self->test_writeResponse_UndefinedResponse();
    $self->test_writeResponse_EmptyOAuthIntegrations();
    $self->test_writeResponse_MissingPartnerClientId();
    $self->test_writeResponse_EmptyMerchantClientId();

    done_testing();
}

sub test_buildRequest_successful {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $PaymentMethodPaypalPlus = $self->_buildPaymentMethodPaypalPlus();

    my $Repository = $self->_createGetMerchantStatus_Repository();
    $Repository->buildRequest($PaymentMethodPaypalPlus);

    my $RequestDTO = $Repository->getRequestDTO();
    is($RequestDTO->trackingId(), 'ShopGUID',"$SubTestName - Test if the ShopGUID was set on the RequestDTO");
    is($RequestDTO->providerPayPalAccountId(), 'Test_PaypalAccount',"$SubTestName - Test if the ProviderPaypalAccountID was set on the RequestDTO");

    my $CredentialDTO = $Repository->getCredentialDTO();
    is($CredentialDTO->isInTestMode(), 'false',"$SubTestName - Test if the TestMode was correctly set");
    is($CredentialDTO->merchantClientId(), undef,"$SubTestName - Test if the merchantClientId was correctly set");
    is($CredentialDTO->providerClientId(), 'TestClientId',"$SubTestName - Test if the TestClientId was correctly set");
    is($CredentialDTO->providerClientSecret(), 'TestClientSecret',"$SubTestName - Test if the TestClientSecret was correctly set");
}
#----------------------------------------------------------------------------------------
sub test_buildRequest_missingProviderPayPalAccountId {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $PaymentMethodPaypalPlus = $self->_buildPaymentMethodPaypalPlus_missingProviderPayPalAccountID();

    my $Repository = $self->_createGetMerchantStatus_Repository();

    throws_ok(
        sub { $Repository->buildRequest($PaymentMethodPaypalPlus); },
        qr/ProviderPayPalAccountId is empty/,
        "$SubTestName - test if ProviderPayPalAccountId is missing"
     );
}
#----------------------------------------------------------------------------------------
sub test_writeResponse_successful {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $PaymentMethodPaypalPlus = $self->_buildPaymentMethodPaypalPlus();
    my $hResponse = {
        'oauth_integrations' => [
            { 'oauth_third_party' => [
                {
                    'partner_client_id'  => 'DefinitelyNotThisOne',
                    'merchant_client_id' => 'SomethingIsWrongWithYou',
                },
                {
                    'partner_client_id'  => 'TestClientId',
                    'merchant_client_id' => 'Test_MerchantClientId',
                }
                ]
            }
         ]
    };

    my $Repository = $self->_createGetMerchantStatus_Repository();
    $Repository->writeResponse($hResponse, $PaymentMethodPaypalPlus);

    is($PaymentMethodPaypalPlus->get('MerchantClientId'), 'Test_MerchantClientId',"$SubTestName - MerchantClientId was correctly set");
}
#----------------------------------------------------------------------------------------
sub test_writeResponse_UndefinedResponse {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $PaymentMethodPaypalPlus = $self->_buildPaymentMethodPaypalPlus();
    my $hResponse = undef;

    my $Repository = $self->_createGetMerchantStatus_Repository();
    throws_ok(
        sub { $Repository->writeResponse($hResponse, $PaymentMethodPaypalPlus); },
        qr/PARAMETER_MISSING/,
        "$SubTestName - Expected exception when response is undefined"
     );
}
#----------------------------------------------------------------------------------------
sub test_writeResponse_EmptyOAuthIntegrations {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $PaymentMethodPaypalPlus = $self->_buildPaymentMethodPaypalPlus();
    my $hResponse = {
        'oauth_integrations' => []
    };

    my $Repository = $self->_createGetMerchantStatus_Repository();
    throws_ok(
        sub { $Repository->writeResponse($hResponse, $PaymentMethodPaypalPlus); },
        qr/PARAMETER_MISSING/,
        "$SubTestName - Expected exception when array is empty"
     );
}
#----------------------------------------------------------------------------------------
sub test_writeResponse_MissingPartnerClientId {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $PaymentMethodPaypalPlus = $self->_buildPaymentMethodPaypalPlus();
    my $hResponse = {
        'oauth_integrations' => [
            { 'oauth_third_party' => [
                {
                    'partner_client_id'  => 'DefinitelyNotThisOne',
                    'merchant_client_id' => 'SomethingIsWrongWithYou',
                },
                {
                    'partner_client_id'  => 'NeitherThisOne',
                    'merchant_client_id' => 'Test_MerchantClientId',
                }
                ]
            }
         ]
    };

    my $Repository = $self->_createGetMerchantStatus_Repository();
    throws_ok(
        sub { $Repository->writeResponse($hResponse, $PaymentMethodPaypalPlus); },
        qr/PARAMETER_MISSING/,
        "$SubTestName - Expected exception when there is no matching ProviderClientId"
     );
}
#----------------------------------------------------------------------------------------
sub test_writeResponse_EmptyMerchantClientId {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $PaymentMethodPaypalPlus = $self->_buildPaymentMethodPaypalPlus();
    my $hResponse = {
        'oauth_integrations' => [
            { 'oauth_third_party' => [
                {
                    'partner_client_id'  => 'DefinitelyNotThisOne',
                    'merchant_client_id' => 'SomethingIsWrongWithYou',
                },
                {
                    'partner_client_id'  => 'TestClientId',
                    'merchant_client_id' => '',
                }
                ]
            }
         ]
    };

    my $Repository = $self->_createGetMerchantStatus_Repository();
    throws_ok(
        sub { $Repository->writeResponse($hResponse, $PaymentMethodPaypalPlus); },
        qr/STRING_EMPTY/,
        "$SubTestName - Expected exception when merchant_client_id is empty"
     );
}


#---------------------------------------------------------------------------
#  Helpers
#---------------------------------------------------------------------------
sub _createGetMerchantStatus_Repository{
    my $self = shift;
    return DE_EPAGES::PayPalPlus::API::Webservice::GetMerchantStatus::Repository->new();
}
#---------------------------------------------------------------------------
sub _buildPaymentMethodPaypalPlus{
    my $self = shift;
    my $ProviderPayPalAccountId = 'Test_PaypalAccount';
    my $PaymentType = $self->_createPaymentType($ProviderPayPalAccountId);
    return $self->_createPaymentMethodPaypalPlus($PaymentType);
}
#---------------------------------------------------------------------------
sub _buildPaymentMethodPaypalPlus_missingProviderPayPalAccountID{
    my $self = shift;
    my $ProviderPayPalAccountId = '';
    my $PaymentType = $self->_createPaymentType($ProviderPayPalAccountId);
    return $self->_createPaymentMethodPaypalPlus($PaymentType);
}
#----------------------------------------------------------------------------------------
sub _createPaymentMethodPaypalPlus {
    my $self = shift;
    my ($PaymentType) = @_;

    my $Shop = $self->_createShop();

    my $hAttributes = {
        'Site' => $Shop,
        'Shop' => $Shop,
        'PaymentType' => $PaymentType,
        'MerchantClientId' => undef,
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus', $hAttributes );
}
#----------------------------------------------------------------------------------------
sub _createPaymentType {
    my $self = shift;
    my ($ProviderPayPalAccountId) = @_;
    my $hAttributes = {
        'ProviderClientId' => 'TestClientId',
        'ProviderClientSecret' => 'TestClientSecret',
        'ProviderPayPalAccountId' => $ProviderPayPalAccountId,
        'PayPalPartnerAttributionId' => 'MyAttributionId',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Payment::API::Object::PaymentType', $hAttributes );
}
#---------------------------------------------------------------------------
sub _createShop {
    my $self = shift;
    my $hAttributes = {
        'IsInternalTestShop' => 0,
        'Alias' => 'MyShopName',
        'GUID' => 'ShopGUID',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Shop::API::Object::Shop', $hAttributes );
}
__PACKAGE__->runScript();
