#==================================================================================================
# package      DE_EPAGES::SIA::t::unit::ResponseParser
# state        public
#--------------------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::SIA::API::Registration::ResponseParser
#==================================================================================================
package DE_EPAGES::SIA::t::unit::ResponseParser;
use base DE_EPAGES::Core::API::Test;

use Test::More;
use Test::MockObject::Extends;

use strict;

use DE_EPAGES::SIA::API::Registration::ResponseParser;

## no critic (ProhibitMagicNumbers)

sub test {
    my $self = shift;

    $self->test_parse_empty_file();
    $self->test_parse_file();
    $self->test_parse_content_line();

    done_testing();
    return;
}

sub test_parse_empty_file {
    my $self = shift;

    my $ResponseParser       = DE_EPAGES::SIA::API::Registration::ResponseParser->new();
    my $MockedResponseParser = Test::MockObject::Extends->new($ResponseParser);

    $MockedResponseParser->mock('_getFileContentLines', sub { return ['010INTERFACE DE MANTENIMIENTO SCNP I+M - SIA                   201610100000000010000  '];});
    my $hLineValues = $MockedResponseParser->parseFile('/tmp/empty.txt');

    is( keys %$hLineValues, 0, 'Skipped header line' );
}

sub test_parse_file {
    my $self = shift;

    my $ResponseParser       = DE_EPAGES::SIA::API::Registration::ResponseParser->new();
    my $MockedResponseParser = Test::MockObject::Extends->new($ResponseParser);

    $MockedResponseParser->mock('_getFileContentLines', sub { return [ "010INTERFACE DE MANTENIMIENTO SCNP I+M - SIA                   201610100000000010000",
                                                                       "0001444a0a5f55dIMN01ACEN000100010046e46341de7d7Max Mustermann                Jena                               27620161006DE 9875686            enEUR1100010000000000000              055720051      DPEC                                                                                                                                                                                                                                                                                                                                    0000NN0000100000 0003N00300180NN  1100000                                                                                                                                                                                                                                                                                                                                                                                                                ADYN               1111111                     0000REGISTRO CORRECTO                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 ",
                                                                       "00012e6372e1a82IMN01ACEN000100010046e46341de7d7Max Mustermann                Jena                               27620161006DE 9875686            enEUR1100010000000000000              055720051      DPEC                                                                                                                                                                                                                                                                                                                                    0000NN0000100000 0003N00300180NN  1100000                                                                                                                                                                                                                                                                                                                                                                                                                ADYN               1111111                     0000REGISTRO CORRECTO                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 "];});
    my $hLineValues = $MockedResponseParser->parseFile('/tmp/foobar.txt');

    is( keys %$hLineValues, 2, 'Found 2 entries in the parsed file' );

    ok( defined $hLineValues->{'444a0a5f55d/tmp/foobar.txt'}, 'Parsed 1st response line from file' );
    ok( defined $hLineValues->{'2e6372e1a82/tmp/foobar.txt'}, 'Parsed 2nd response line from file' );

    is( $hLineValues->{'444a0a5f55d/tmp/foobar.txt'}->{'WorkOrderID'}, '444a0a5f55d', 'Correct 1st WorkOrderID from file' );
    is( $hLineValues->{'2e6372e1a82/tmp/foobar.txt'}->{'WorkOrderID'}, '2e6372e1a82', 'Correct 2nd WorkOrderID from file' );
    is( $hLineValues->{'444a0a5f55d/tmp/foobar.txt'}->{'ErrorCode'}, '0000', 'Correct 1st ErrorCode from file' );
    is( $hLineValues->{'2e6372e1a82/tmp/foobar.txt'}->{'ErrorCode'}, '0000', 'Correct 2nd ErrorCode from file' );
}

sub test_parse_content_line {
    my $self = shift;

    my $ResponseParser = DE_EPAGES::SIA::API::Registration::ResponseParser->new();

    my $ResponseLine1 = "0001444a0a5f55dIMN01ACEN000100010046e46341de7d7Max Mustermann                Jena                               27620161006DE 9875686            enEUR1100010000000000000              055720051      DPEC                                                                                                                                                                                                                                                                                                                                    0000NN0000100000 0003N00300180NN  1100000                                                                                                                                                                                                                                                                                                                                                                                                                ADYN               1111111                     0000REGISTRO CORRECTO                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 ";
    my $ResponseLine2 = "00012e6372e1a82IMN01ACEN000100010046e46341de7d7Max Mustermann                Jena                               27620161006DE 9875686            enEUR1100010000000000000              055720051      DPEC                                                                                                                                                                                                                                                                                                                                    0000NN0000100000 0003N00300180NN  1100000                                                                                                                                                                                                                                                                                                                                                                                                                ADYN               1111111                     0000REGISTRO CORRECTO                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 ";

    my $hLineValues = $ResponseParser->parseContentLine($ResponseLine1);

    is( $hLineValues->{'WorkOrderID'}, '444a0a5f55d', 'ResponseLine1 WorkOrderID is correct' );
    is( $hLineValues->{'ErrorCode'},   '0000',        'ResponseLine1 ErrorCode is correct' );

    $hLineValues = $ResponseParser->parseContentLine($ResponseLine2);
    is( $hLineValues->{'WorkOrderID'}, '2e6372e1a82', 'ResponseLine2 WorkOrderID is correct' );
    is( $hLineValues->{'ErrorCode'},   '0000',        'ResponseLine2 ErrorCode is correct' );

    my $ResponseLineWindowsLineEnding = "0001444a0a5f55dIMN01ACEN000100010046e46341de7d7Max Mustermann                Jena                               27620161006DE 9875686            enEUR1100010000000000000              055720051      DPEC                                                                                                                                                                                                                                                                                                                                    0000NN0000100000 0003N00300180NN  1100000                                                                                                                                                                                                                                                                                                                                                                                                                ADYN               1111111                     0000REGISTRO CORRECTO                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 \r\n";
    my $ResponseLineLinuxLineEnding   = "0001444a0a5f55dIMN01ACEN000100010046e46341de7d7Max Mustermann                Jena                               27620161006DE 9875686            enEUR1100010000000000000              055720051      DPEC                                                                                                                                                                                                                                                                                                                                    0000NN0000100000 0003N00300180NN  1100000                                                                                                                                                                                                                                                                                                                                                                                                                ADYN               1111111                     0000REGISTRO CORRECTO                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 \n";
    my $ResponseLineOldUnixLineEnding = "0001444a0a5f55dIMN01ACEN000100010046e46341de7d7Max Mustermann                Jena                               27620161006DE 9875686            enEUR1100010000000000000              055720051      DPEC                                                                                                                                                                                                                                                                                                                                    0000NN0000100000 0003N00300180NN  1100000                                                                                                                                                                                                                                                                                                                                                                                                                ADYN               1111111                     0000REGISTRO CORRECTO                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 \n\r";

    $hLineValues = $ResponseParser->parseContentLine($ResponseLineWindowsLineEnding);
    is( $hLineValues->{'WorkOrderID'}, '444a0a5f55d', 'ResponseLineWindowsLineEnding WorkOrderID is correct' );
    is( $hLineValues->{'ErrorCode'},   '0000',        'ResponseLineWindowsLineEnding ErrorCode is correct' );

    $hLineValues = $ResponseParser->parseContentLine($ResponseLineLinuxLineEnding);
    is( $hLineValues->{'WorkOrderID'}, '444a0a5f55d', 'ResponseLineLinuxLineEnding WorkOrderID is correct' );
    is( $hLineValues->{'ErrorCode'},   '0000',        'ResponseLineLinuxLineEnding ErrorCode is correct' );

    $hLineValues = $ResponseParser->parseContentLine($ResponseLineOldUnixLineEnding);
    is( $hLineValues->{'WorkOrderID'}, '444a0a5f55d', 'ResponseLineOldUnixLineEnding WorkOrderID is correct' );
    is( $hLineValues->{'ErrorCode'},   '0000',        'ResponseLineOldUnixLineEnding ErrorCode is correct' );

    return;
}

__PACKAGE__->runScript();
1;
